local colors = {
  SKINCOLOR_RED, 
  SKINCOLOR_ORANGE,
  SKINCOLOR_YELLOW,
  SKINCOLOR_GREEN,
  SKINCOLOR_AZURE, 
  SKINCOLOR_BLUE,
  SKINCOLOR_PURPLE
}

rawset(_G, "tableIndex", function(tbl, value)
    for k, v in pairs(tbl) do
        if v == value then
            return k
        end
    end
    return nil
end)

addHook("PlayerSpawn", function(p)
    p.sparkTable = {}
    p.inv = nil
end)

local function P_SpawnRotatingSparkles(player)
    local mo = player.mo
    local angle = 0
    local radius = 128*player.mo.scale
    local sparkTable = player.sparkTable or {}
    
    if player.powers[pw_super] then
        return
    end
    
    if player.powers[pw_invulnerability] and player.mo.skin == UG then
        if #sparkTable >= 4 then
            return
        end
        
        for i = 1, 4 - #sparkTable do
            local x = FixedMul(radius, cos(angle))
            local y = FixedMul(radius, sin(angle))
            local spark = P_SpawnMobjFromMobj(mo, x, y, 0, MT_SUPERSPARK)
            spark.angle = angle
            spark.radius = radius
            spark.yup = true
            spark.p = player
            spark.rollangle = player.mo.rollangle
            spark.scale = mo.scale
            spark.colorized = true
            local colorIndex = (leveltime % #colors) + 1
            spark.color = colors[colorIndex]
            angle = angle + ANGLE_90
            spark.angle = angle
            table.insert(sparkTable, spark)
        end
    else 
        for i, spark in ipairs(sparkTable) do
            if spark.valid then
                P_RemoveMobj(spark)
            end
            table.remove(sparkTable, i)
            break
        end
    end
    
    player.sparkTable = sparkTable
end

addHook("PlayerThink", function(p)
	if p.mo and p.mo.valid
		and p.mo.skin ~= UG
		p.sparkTable = {}
	end
end)
		

addHook("ThinkFrame", function()
    for p in players.iterate do
        if p.mo and p.mo.valid and p.powers[pw_invulnerability] and not p.powers[pw_super] and p.mo.skin == UG then
            local m = p.mo
            if not p.inv then
                p.inv = P_SpawnMobj(m.x, m.y, m.z, MT_OVERLAY)
                p.inv.target = p.mo
                p.inv.sprite = SPR_PLAY
                p.inv.skin = m.skin
				P_SpawnRotatingSparkles(p)
                p.inv.sprite2 = m.sprite2
                p.inv.state = p.mo.state
            else
                local ia = p.inv
                local gho = P_SpawnGhostMobj(m)
                gho.scale = m.scale
				P_SpawnRotatingSparkles(p)
                gho.colorized = true
                gho.color = p.inv.color
                gho.skin = m.skin
                gho.frame = m.frame
                gho.sprite = SPR_PLAY
                gho.sprite2 = m.sprite2
                P_MoveOrigin(ia, m.x, m.y, m.z)
                gho.tics = 3
                gho.destscale = m.scale * 6 / 5
                ia.target = p.mo
                p.inv.frame = m.frame
                p.inv.colorized = true
                ia.skin = m.skin
                p.inv.angle = p.drawangle
                p.inv.rollangle = m.rollangle
                p.inv.sprite2 = m.sprite2
				P_SpawnRotatingSparkles(p)
                if m.flags2 & MF2_DONTDRAW
                    ia.flags2 = $|MF2_DONTDRAW
				else
					ia.flags2 = $ & ~MF2_DONTDRAW
                end
                ia.frame = m.frame
                p.inv.spritexscale = m.spritexscale
                p.inv.spriteyscale = m.spriteyscale
                p.inv.dispoffset = 30
                local colorIndex = (leveltime % #colors) + 1
                p.inv.color = colors[colorIndex]
            end
        elseif p.inv and p.inv.valid then
            P_RemoveMobj(p.inv)
            p.inv = nil
        end
    end
end)

addHook("MobjThinker", function(s)
    if s and s.valid and s.yup then
        local player = s.p
        local index = tableIndex(player.sparkTable, s)
        if player.powers[pw_invulnerability] < 1 and index then
            table.remove(player.sparkTable, index)
            P_RemoveMobj(s)
        end
    end
end, MT_SUPERSPARK)



addHook("MobjRemoved", function(s)
    if s.p and s.p.valid and s.p.powers[pw_invulnerability] and s.p.mo.skin == UG then
        local player = s.p
        local sparkTable = player.sparkTable
        local index = tableIndex(sparkTable, s)
        if player.powers[pw_super] then return end
        if index then
            if player.mo.skin == UG then
                local x = s.x
                local y = s.y
                local z = s.z
                local newSpark = P_SpawnMobjFromMobj(s, 0, 0, 0, MT_SUPERSPARK)
				
                newSpark.angle = s.angle
                newSpark.radius = s.radius
                newSpark.yup = true
				
                newSpark.p = player
                newSpark.rollangle = 0
                newSpark.scale = s.scale
                newSpark.colorized = true
                local colorIndex = (leveltime % #colors) + 1
                newSpark.color = colors[colorIndex]
                table.insert(sparkTable, index, newSpark)
            end
            table.remove(sparkTable, index + 1)
        end
    end
end, MT_SUPERSPARK)

addHook("MobjThinker", function(s)
    if s and s.valid and s.yup then
        local angle = s.angle
        local radius = 128*s.p.mo.scale
        local player = s.p
		local pmo = s.p.mo
        local x = FixedMul(radius, cos(angle))
        local y = FixedMul(radius, sin(angle))
		local z = pmo.z
		if pmo.eflags & MFE_VERTICALFLIP
			z = pmo.z + pmo.height
		else
			z = pmo.z
		end
        P_MoveOrigin(s, player.mo.x + x, player.mo.y + y, z)
        s.angle = $ + FixedAngle(2*FU)
        s.scale = player.mo.scale
		if pmo.eflags & MFE_VERTICALFLIP
			s.eflags = $|MFE_VERTICALFLIP
		else
			s.eflags = $ & ~MFE_VERTICALFLIP
		end
        s.rollangle = player.mo.rollangle
        local guh = P_SpawnGhostMobj(s)
        guh.tics = 4
    end
end, MT_SUPERSPARK)

